@@ -53,7 +53,7 @@
 #include <math.h>
 #include "object.h"
 #include "nstring.h"
-#include "jstring.h"
+#include "jnstring.h"
 #include "mathfn.h"
 #include "mathcode.h"
 #include "gra.h"
