@@ -1,5 +1,5 @@
 /* 
- * $Id: mathcode.c,v 1.6 2002/07/06 08:51:42 isizaka Exp isizaka $
+ * $Id: mathcode.c,v 1.8 2009/04/03 15:37:52 isizaka Exp isizaka $
  * 
  * This file is part of "Ngraph for X11".
  * 
@@ -24,6 +24,9 @@
 /**
  *
  * $Log: mathcode.c,v $
+ * Revision 1.8  2009/04/03 15:37:52  isizaka
+ * add sign of '+'
+ *
  * Revision 1.6  2002/07/06 08:51:42  isizaka
  * change to GPL.
  *
@@ -146,8 +149,7 @@ struct {
 
 /*
 used character code:
-
-=XYZ#|+-/*\^()!,"@
+=XYZ#|+-/*\^(){}[]!,"@&%$~?_`
 8x
 9x
 b0 b1 b2 b3
@@ -571,7 +573,7 @@ void mathexpand(int pre,int *oppo,int *numpo,int *ifpo,int *forpo,
     if (strchr("+-*/\\^",opbuf[*oppo])!=NULL) argnum=2;
     else argnum=argbuf[*oppo];
     if (opbuf[*oppo]=='?') {
-      if (numbuf[*numpo-2]!=0) {
+      if ((numstat[*numpo-2]==MNOERR) && (numbuf[*numpo-2]!=0)) {
         numbuf[*numpo-2]=numbuf[*numpo-1];
         numstat[*numpo-2]=numstat[*numpo-1];
       } else {
@@ -621,6 +623,9 @@ void mathexpand(int pre,int *oppo,int *numpo,int *ifpo,int *forpo,
               else numbuf[*numpo-1]=exp(x);
             }
             break;
+          case '`':
+            numbuf[*numpo]=numbuf[*numpo];
+            break;
           case '+': 
             numbuf[*numpo-1]=numbuf[*numpo-1]+numbuf[*numpo];
             break;
@@ -1058,6 +1063,7 @@ int calculate(char *code,
     po=0;
     while (code[po]!='_') {
         if ((code[po]=='-') && !numeric) code[po]='&';
+        if ((code[po]=='+') && !numeric) code[po]='`';
         switch (code[po]) {
         case '#':
           if (numeric) return MSERR;
@@ -1318,6 +1324,7 @@ int calculate(char *code,
           numeric=TRUE;
           break;
         case '&':
+        case '`':
           if (numeric) return MSERR;
           oppo++;
           if (oppo>=20) return MSERR;
