@@ -1,21 +1,41 @@
-/**
- *
- * $Id: oaxis.c,v 1.7 1999/10/15 16:32:12 isizaka Exp isizaka $
- *
- * Copyright (C) 1997, Satoshi ISHIZAKA.
- *
- * This file is part of Ngraph.
- *
- * This is free software; you can redistribute it and/or modify it.
- * However, it is prohibited to compile this on the "Windows" environment.
- *
- * Original author: Satoshi ISHIZAKA
- *                  isizaka@msa.biglobe.ne.jp
- **/
+/* 
+ * $Id: oaxis.c,v 1.11 2003/02/16 12:42:14 isizaka Exp isizaka $
+ * 
+ * This file is part of "Ngraph for X11".
+ * 
+ * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
+ * 
+ * "Ngraph for X11" is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * "Ngraph for X11" is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * 
+ */
 
 /**
  *
  * $Log: oaxis.c,v $
+ * Revision 1.11  2003/02/16 12:42:14  isizaka
+ * for release 6.13.18
+ *
+ * Revision 1.10  2002/07/06 08:51:42  isizaka
+ * change to GPL.
+ *
+ * Revision 1.9  2001/03/23 12:15:31  isizaka
+ * for 6.3.13
+ *
+ * Revision 1.8  2000/10/28 14:46:43  isizaka
+ * axisautoscale(): inc is preserved if it is set.
+ *
  * Revision 1.7  1999/10/15 16:32:12  isizaka
  * add 'numformat' for "+/-0".
  *
@@ -586,8 +606,8 @@ int axisbbox2(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
   _getobj(obj,"direction",inst,&direction);
   if ((array==NULL) && ((array=arraynew(sizeof(int)))==NULL)) return 1;
   dir=direction/18000.0*MPI;
-  x1=x0+round(length*cos(dir));
-  y1=y0-round(length*sin(dir));
+  x1=x0+nround(length*cos(dir));
+  y1=y0-nround(length*sin(dir));
   maxx=minx=x0;
   maxy=miny=y0;
   arrayadd(array,&x0);
@@ -1326,7 +1346,7 @@ int axiszoom(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
   return 0;
 }
 
-int numformat(char *num,char *format,double a)
+void numformat(char *num,char *format,double a)
 {
   int i,j,len,ret;
   char *s;
@@ -1419,8 +1439,8 @@ int axisdraw(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
   sdata=arraydata(style);
 
   dir=direction/18000.0*MPI;
-  x1=x0+round(length*cos(dir));
-  y1=y0-round(length*sin(dir));
+  x1=x0+nround(length*cos(dir));
+  y1=y0-nround(length*sin(dir));
 
   GRAregion(GC,&lm,&tm,&w,&h,&zoom);
   GRAview(GC,0,0,w*10000.0/zoom,h*10000.0/zoom,clip);
@@ -1436,24 +1456,24 @@ int axisdraw(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
   if ((arrow==2) || (arrow==3)) {
     dx=-cos(dir);
     dy=sin(dir);
-    ap[0]=round(x0-dy*awid);
-    ap[1]=round(y0+dx*awid);
-    ap[2]=round(x0+dx*alen);
-    ap[3]=round(y0+dy*alen);
-    ap[4]=round(x0+dy*awid);
-    ap[5]=round(y0-dx*awid);
+    ap[0]=nround(x0-dy*awid);
+    ap[1]=nround(y0+dx*awid);
+    ap[2]=nround(x0+dx*alen);
+    ap[3]=nround(y0+dy*alen);
+    ap[4]=nround(x0+dy*awid);
+    ap[5]=nround(y0-dx*awid);
     GRAlinestyle(GC,0,NULL,1,0,0,1000);
     GRAdrawpoly(GC,3,ap,1);
   }
   if ((arrow==1) || (arrow==3)) {
     dx=cos(dir);
     dy=-sin(dir);
-    ap[0]=round(x1-dy*awid);
-    ap[1]=round(y1+dx*awid);
-    ap[2]=round(x1+dx*alen);
-    ap[3]=round(y1+dy*alen);
-    ap[4]=round(x1+dy*awid);
-    ap[5]=round(y1-dx*awid);
+    ap[0]=nround(x1-dy*awid);
+    ap[1]=nround(y1+dx*awid);
+    ap[2]=nround(x1+dx*alen);
+    ap[3]=nround(y1+dy*alen);
+    ap[4]=nround(x1+dy*awid);
+    ap[5]=nround(y1-dx*awid);
     GRAlinestyle(GC,0,NULL,1,0,0,1000);
     GRAdrawpoly(GC,3,ap,1);
   }
@@ -1461,20 +1481,20 @@ int axisdraw(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
   if ((wave==2) || (wave==3)) {
     dx=cos(dir);
     dy=sin(dir);
-    wx[0]=round(x0-dy*wlength);
-    wx[1]=round(x0-dy*0.5*wlength-dx*0.25*wlength);
+    wx[0]=nround(x0-dy*wlength);
+    wx[1]=nround(x0-dy*0.5*wlength-dx*0.25*wlength);
     wx[2]=x0;
-    wx[3]=round(x0+dy*0.5*wlength+dx*0.25*wlength);
-    wx[4]=round(x0+dy*wlength);
+    wx[3]=nround(x0+dy*0.5*wlength+dx*0.25*wlength);
+    wx[4]=nround(x0+dy*wlength);
     if (spline(ww,wx,wxc1,wxc2,wxc3,5,SPLCND2NDDIF,SPLCND2NDDIF,0,0)) {
       error(obj,ERRAXISSPL);
       goto exit;
     }
-    wy[0]=round(y0-dx*wlength);
-    wy[1]=round(y0-dx*0.5*wlength+dy*0.25*wlength);
+    wy[0]=nround(y0-dx*wlength);
+    wy[1]=nround(y0-dx*0.5*wlength+dy*0.25*wlength);
     wy[2]=y0;
-    wy[3]=round(y0+dx*0.5*wlength-dy*0.25*wlength);
-    wy[4]=round(y0+dx*wlength);
+    wy[3]=nround(y0+dx*0.5*wlength-dy*0.25*wlength);
+    wy[4]=nround(y0+dx*wlength);
     if (spline(ww,wy,wyc1,wyc2,wyc3,5,SPLCND2NDDIF,SPLCND2NDDIF,0,0)) {
       error(obj,ERRAXISSPL);
       goto exit;
@@ -1494,20 +1514,20 @@ int axisdraw(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
   if ((wave==1) || (wave==3)) {
     dx=cos(dir);
     dy=sin(dir);
-    wx[0]=round(x1-dy*wlength);
-    wx[1]=round(x1-dy*0.5*wlength-dx*0.25*wlength);
+    wx[0]=nround(x1-dy*wlength);
+    wx[1]=nround(x1-dy*0.5*wlength-dx*0.25*wlength);
     wx[2]=x1;
-    wx[3]=round(x1+dy*0.5*wlength+dx*0.25*wlength);
-    wx[4]=round(x1+dy*wlength);
+    wx[3]=nround(x1+dy*0.5*wlength+dx*0.25*wlength);
+    wx[4]=nround(x1+dy*wlength);
     if (spline(ww,wx,wxc1,wxc2,wxc3,5,SPLCND2NDDIF,SPLCND2NDDIF,0,0)) {
       error(obj,ERRAXISSPL);
       goto exit;
     }
-    wy[0]=round(y1-dx*wlength);
-    wy[1]=round(y1-dx*0.5*wlength+dy*0.25*wlength);
+    wy[0]=nround(y1-dx*wlength);
+    wy[1]=nround(y1-dx*0.5*wlength+dy*0.25*wlength);
     wy[2]=y1;
-    wy[3]=round(y1+dx*0.5*wlength-dy*0.25*wlength);
-    wy[4]=round(y1+dx*wlength);
+    wy[3]=nround(y1+dx*0.5*wlength-dy*0.25*wlength);
+    wy[4]=nround(y1+dx*wlength);
     if (spline(ww,wy,wyc1,wyc2,wyc3,5,SPLCND2NDDIF,SPLCND2NDDIF,0,0)) {
       error(obj,ERRAXISSPL);
       goto exit;
@@ -1712,7 +1732,7 @@ numbering:
     if (alocal.atype==AXISINVERSE) {
       if (step==0) {
         if (count==0) n=1;
-        else n=round(pow(10.0,(double )(int )(log10((double )count))));
+        else n=nround(pow(10.0,(double )(int )(log10((double )count))));
         if (n!=1) n--;
         if (count>=18*n) step=9*n;
         else if (count>=6*n) step=3*n;
@@ -1725,13 +1745,13 @@ numbering:
     } else {
       if (step==0) {
         if (count==0) n=1;
-        else n=round(pow(10.0,(double )(int )(log10(count*0.5))));
+        else n=nround(pow(10.0,(double )(int )(log10(count*0.5))));
         if (count>=10*n) step=5*n;
         else if (count>=5*n) step=2*n;
         else step=n;
       }
       if (begin==0)
-        begin=round(fabs((roundmin(alocal.posst,alocal.dposl)
+        begin=nround(fabs((roundmin(alocal.posst,alocal.dposl)
                         -roundmin(alocal.posst,alocal.dposm))/alocal.dposm))
              % step+1;
     }
@@ -2007,14 +2027,11 @@ exit:
 int axisclear(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
 {
   double min,max,inc;
-  int div;
 
   min=max=inc=0;
-  div=0;
   if (_putobj(obj,"min",inst,&min)) return 1;
   if (_putobj(obj,"max",inst,&max)) return 1;
   if (_putobj(obj,"inc",inst,&inc)) return 1;
-  if (_putobj(obj,"div",inst,&div)) return 1;
   return 0;
 }
 
@@ -2091,8 +2108,8 @@ int axisadjust(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
   if (first) return 0;
   x=-sin(dir1)*(gx0-posx1)-cos(dir1)*(gy0-posy1);
   x=x/(-cos(dir)*sin(dir1)+sin(dir)*cos(dir1));
-  posx1=round(posx1+x*cos(dir));
-  posy1=round(posy1-x*sin(dir));
+  posx1=nround(posx1+x*cos(dir));
+  posy1=nround(posy1-x*sin(dir));
   if (_putobj(obj,"x",inst,&posx1)) return 1;
   if (_putobj(obj,"y",inst,&posy1)) return 1;
   _getobj(obj,"bbox",inst,&array);
@@ -2161,7 +2178,7 @@ int axischangescale(struct objlist *obj,char *inst,
     inc=scale(max-min);
   }
   if ((type!=1) && (inc==0)) inc=1;
-  if (type==1) inc=round(inc);
+  if (type==1) inc=nround(inc);
   inc=fabs(inc);
   if (type==1) {
     min=pow(10.0,min);
@@ -2278,17 +2295,23 @@ int axisautoscale(struct objlist *obj,char *inst,char *rval,
 {
   char *fileobj;
   int room;
-  double min,max;
-  char *argv2[4];
+  double omin,omax,oinc;
+  double min,max,inc;
 
   fileobj=(char *)argv[2];
   room=*(int *)argv[3];
   if (axisautoscalefile(obj,inst,fileobj,&min,&max)==0) {
-    argv2[0]=(char *)&min;
-    argv2[1]=(char *)&max;
-    argv2[2]=(char *)&room;
-    argv2[3]=NULL;
-    _exeobj(obj,"scale",inst,3,argv2);
+    axischangescale(obj,inst,&min,&max,&inc,room);
+    _getobj(obj,"min",inst,&omin);
+    _getobj(obj,"max",inst,&omax);
+    _getobj(obj,"inc",inst,&oinc);
+    if (omin==omax) {
+      if (_putobj(obj,"min",inst,&min)) return 1;
+      if (_putobj(obj,"max",inst,&max)) return 1;
+    }
+    if (oinc==0) {
+      if (_putobj(obj,"inc",inst,&inc)) return 1;
+    }
   }
   return 0;
 }
@@ -2961,7 +2984,7 @@ struct objtable axis[TBLNUM] = {
   {"bbox",NIAFUNC,NREAD|NEXEC,axisbbox,"",0},
   {"move",NVFUNC,NREAD|NEXEC,axismove,"ii",0},
   {"change",NVFUNC,NREAD|NEXEC,axischange,"iii",0},
-  {"zoom",NVFUNC,NREAD|NEXEC,axiszoom,"iii",0},
+  {"zooming",NVFUNC,NREAD|NEXEC,axiszoom,"iii",0},
   {"match",NBFUNC,NREAD|NEXEC,axismatch,"iiiii",0},
   {"coordinate",NDFUNC,NREAD|NEXEC,axiscoordinate,"ii",0},
   {"tight",NVFUNC,NREAD|NEXEC,axistight,NULL,0},
