@@ -1,5 +1,5 @@
 /* 
- * $Id: object.c,v 1.6 2003/02/16 12:42:14 isizaka Exp isizaka $
+ * $Id: object.c,v 1.7 2006/07/20 16:38:33 isizaka Exp isizaka $
  * 
  * This file is part of "Ngraph for X11".
  * 
@@ -24,8 +24,8 @@
 /**
  *
  * $Log: object.c,v $
- * Revision 1.6  2003/02/16 12:42:14  isizaka
- * for release 6.13.18
+ * Revision 1.7  2006/07/20 16:38:33  isizaka
+ * 'offset' data alignment (thanks to  Mr. Mook and Mr. tanaka)
  *
  * Revision 1.5  2002/07/06 08:51:42  isizaka
  * change to GPL.
@@ -56,6 +56,9 @@
 #include "object.h"
 #include "mathcode.h"
 #include "mathfn.h"
+#ifdef WINDOWS
+#include <math.h>
+#endif
 
 #ifdef DEBUG
 #ifdef WINDOWS
@@ -355,6 +358,7 @@ void *memrealloc(void *ptr,size_t size)
       }
       if (plcur==NULL) {
         printfconsole("*%p\n",ptr);
+        sleep(30);
         exit(1);
       }
       if (plprev==NULL) memallocroot=plcur->next;
@@ -870,6 +874,7 @@ void *addobject(char *name,char *alias,char *parentname,char *ver,
   objnew->doneproc=doneproc;
   if (parent==NULL) offset=0;
   else offset=parent->size;
+  if (offset%ALIGNSIZE !=0) offset=offset+(ALIGNSIZE-offset%ALIGNSIZE);
   for (i=0;i<tblnum;i++) {
     table[i].offset=offset;
     switch (table[i].type) {
@@ -889,6 +894,7 @@ void *addobject(char *name,char *alias,char *parentname,char *ver,
     default:
       offset+=sizeof(void *);
     }
+    if (offset%ALIGNSIZE !=0) offset=offset+(ALIGNSIZE-offset%ALIGNSIZE);
     if (table[i].attrib & NEXEC) table[i].attrib&=~NWRITE;
   }
   objnew->size=offset;
