@@ -60,7 +60,7 @@
 #include "ioutil.h"
 #include "gra.h"
 #include "nstring.h"
-#include "jstring.h"
+#include "jnstring.h"
 #include "mathfn.h"
 #include "config.h"
 
