@@ -63,7 +63,7 @@
 #endif
 #include "ngraph.h"
 #include "mathfn.h"
-#include "jstring.h"
+#include "jnstring.h"
 #include "ntime.h"
 #include "object.h"
 #include "gra.h"
@@ -1667,7 +1667,7 @@ int prmload(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
       if (buf[0]=='[') {
         if ((s2=strchr(buf,']'))!=NULL) {
           if ((filename=memalloc(s2-buf))==NULL) goto errexit;
-          strncpy(filename,buf+1,s2-buf-1);
+          strncpy(filename,buf+1,s2-1-buf);
           filename[s2-buf-1]='\0';
           filename=pathconv(filename,ignorepath);
           if (access(filename,04)!=0) mkdata=TRUE;
