@@ -63,7 +63,7 @@
 #include "ioutil.h"
 #include "gra.h"
 #include "nstring.h"
-#include "jstring.h"
+#include "jnstring.h"
 #include "mathfn.h"
 #include "config.h"
 
@@ -115,9 +115,13 @@ struct fontlocal {
   Font font;
   XFontStruct *fontstruct;
   int fontsize,fonttype,fontdir;
-  int iso8859;
+  int iso;
 };
 
+#define ISOOTHER 0
+#define ISO8859 1
+#define ISO10646 2
+
 struct x11local {
   struct objlist *obj;
   char *inst;
@@ -145,6 +149,7 @@ struct x11local {
   int PaperWidth,PaperHeight;
   int backingstore;
   int minus_hyphen;
+  int font_slant;
 };
 
 int x11loadconfig(struct objlist *obj,struct x11local *x11local)
@@ -234,6 +239,12 @@ int x11loadconfig(struct objlist *obj,struct x11local *x11local)
       val=strtol(f1,&endptr,10);
       if (endptr[0]=='\0') x11local->minus_hyphen=val;
       memfree(f1);
+    } else if (strcmp(tok,"font_slant")==0) {
+      f1=getitok2(&s2,&len," \x09,");
+      val=strtol(f1,&endptr,10);
+      if ((endptr[0]=='\0') && (0<=val) && (val<=60))
+        x11local->font_slant=val;
+      memfree(f1);
     }
     memfree(tok);
     memfree(str);
@@ -388,6 +399,7 @@ int x11init(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
   x11local->cdepth=X11COLORDEPTH;
   x11local->backingstore=FALSE;
   x11local->minus_hyphen=TRUE;
+  x11local->font_slant=15;
   if (x11loadconfig(obj,x11local)) goto errexit;
   if (x11local->windpi<1) x11local->windpi=1;
   if (x11local->windpi>2540) x11local->windpi=2540;
@@ -774,6 +786,7 @@ int x11loadfont(Display *disp,GC gc,struct x11local *x11local,
   XFontStruct *newfontstruct;
   int store;
   int twobyte=FALSE,type;
+  double ftan;
 
   fontcashfind=-1;
   for (i=0;i<x11local->loadfont;i++) {
@@ -821,9 +834,10 @@ int x11loadfont(Display *disp,GC gc,struct x11local *x11local,
   fontname2[fsp-fontname+1]='\0';
   fontname3[fsp-fontname+1]='\0';
   if ((type==ITALIC) || (type==BOLDITALIC)) {
+    ftan=tan(x11local->font_slant*MPI/180.0);
     sprintf(fontmatrix,"[%.3e %.3e %.3e %.3e]",
             size*fcos,size*fsin,
-           -size*fsin+size*fcos*0.36,size*fcos+size*fsin*0.36);
+           -size*fsin+size*fcos*ftan,size*fcos+size*fsin*ftan);
   } else {
     sprintf(fontmatrix,"[%.3e %.3e %.3e %.3e]",
             size*fcos,size*fsin,-size*fsin,size*fcos);
@@ -872,9 +886,11 @@ int x11loadfont(Display *disp,GC gc,struct x11local *x11local,
     (x11local->font[store]).fonttype=type;
     (x11local->font[store]).fontdir=dir;
     if (strcmp(fname2[0]+strlen(fname2[0])-9,"iso8859-1")==0)
-      (x11local->font[store]).iso8859=TRUE;
+      (x11local->font[store]).iso=ISO8859;
+    else if (strcmp(fname2[0]+strlen(fname2[0])-10,"iso10646-1")==0)
+      (x11local->font[store]).iso=ISO10646;
     else
-      (x11local->font[store]).iso8859=FALSE;
+      (x11local->font[store]).iso=ISOOTHER;
     if (top) XSetFont(disp,gc,(x11local->font[store]).font);
     x11local->loadfont++;
     if (fname2!=NULL) XFreeFontNames(fname2);
@@ -886,6 +902,38 @@ int x11loadfont(Display *disp,GC gc,struct x11local *x11local,
   return -1;
 }
 
+void x11XDrawString(struct fontlocal font,Display *Disp,Drawable win,GC gc,
+                    int x,int y,char *ch)
+{
+  XChar2b kanji[1];
+
+  if ((font.iso==ISO10646) && (ch[0]=='-')) {
+    kanji[0].byte1=0x22;
+    kanji[0].byte2=0x12;
+    XDrawString16(Disp,win,gc,x,y,kanji,1);
+  } else {
+    XDrawString(Disp,win,gc,x,y,ch,1);
+  }
+}
+
+int x11XTextWidth(struct fontlocal font,int minus_hyphen,char *ch)
+{
+  XChar2b kanji[1];
+
+  if (font.fontstruct==NULL) {
+    return font.fontsize;
+  } else if ((font.iso==ISO10646) && (ch[0]=='-')) {
+    kanji[0].byte1=0x22;
+    kanji[0].byte2=0x12;
+    return XTextWidth16(font.fontstruct,kanji,1);
+  } else {
+    if ((font.iso==ISO8859) && minus_hyphen && (ch[0]=='-')) {
+      ch[0]='+';
+    }
+    return XTextWidth(font.fontstruct,ch,1);
+  }
+}
+
 int x11_output(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
 {
   struct x11local *x11local;
@@ -1128,32 +1176,24 @@ int x11_output(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
         ch[0]=cstr[i];
         i++;
       }
-      XDrawString(Disp,x11local->win,x11local->gc,
+      x11XDrawString(x11local->font[0],Disp,x11local->win,x11local->gc,
                   dot2pixelx(x11local,nround(x0)),
-                  dot2pixely(x11local,nround(y0)),ch,1);
+                  dot2pixely(x11local,nround(y0)),ch);
       if ((x11local->font[0].fonttype==BOLD) 
        || (x11local->font[0].fonttype==BOLDITALIC)) {
         bs=x11local->fontsize*0.018;
-        XDrawString(Disp,x11local->win,x11local->gc,
+        x11XDrawString(x11local->font[0],Disp,x11local->win,x11local->gc,
           dot2pixelx(x11local,nround(x0+bs)),
-          dot2pixely(x11local,nround(y0)),ch,1); 
-        XDrawString(Disp,x11local->win,x11local->gc,
+          dot2pixely(x11local,nround(y0)),ch); 
+        x11XDrawString(x11local->font[0],Disp,x11local->win,x11local->gc,
           dot2pixelx(x11local,nround(x0)),
-          dot2pixely(x11local,nround(y0-bs)),ch,1); 
-        XDrawString(Disp,x11local->win,x11local->gc,
+          dot2pixely(x11local,nround(y0-bs)),ch); 
+        x11XDrawString(x11local->font[0],Disp,x11local->win,x11local->gc,
           dot2pixelx(x11local,nround(x0+bs)),
-          dot2pixely(x11local,nround(y0-bs)),ch,1); 
-      }
-      if ((x11local->font[0]).fontstruct==NULL) {
-        fontwidth=pixel2dot(x11local,(x11local->font[0]).fontsize);
-      } else {
-        if ((x11local->font[0]).iso8859 && x11local->minus_hyphen
-        && (ch[0]=='-')) {
-          ch[0]='+';
-        }
-        fontwidth=pixel2dot(x11local,
-                  XTextWidth((x11local->font[0]).fontstruct,ch,1));
+          dot2pixely(x11local,nround(y0-bs)),ch); 
       }
+      fontwidth=pixel2dot(x11local,
+                x11XTextWidth(x11local->font[0],x11local->minus_hyphen,ch));
       x0+=(fontwidth+x11local->fontspace)*x11local->fontcos;
       y0-=(fontwidth+x11local->fontspace)*x11local->fontsin;
     }
@@ -1238,11 +1278,8 @@ int x11_charwidth(struct objlist *obj,char *inst,char *rval,
         kanji[0].byte2=jis & 0xff;
         *(int *)rval=pixel2dot(x11local,XTextWidth16(fontstruct,kanji,1));
       } else {
-        if ((x11local->font[cashpos]).iso8859 && x11local->minus_hyphen
-        && (ch[0]=='-')) {
-          ch[0]='+';
-        }
-        *(int *)rval=pixel2dot(x11local,XTextWidth(fontstruct,ch,1));
+        *(int *)rval=pixel2dot(x11local,x11XTextWidth(x11local->font[cashpos],
+                                                  x11local->minus_hyphen,ch));
       }
     }
   } else {
