@@ -52,7 +52,7 @@
 #include "ngraph.h"
 #include "object.h"
 #include "nstring.h"
-#include "jstring.h"
+#include "jnstring.h"
 #include "ioutil.h"
 
 #include "x11gui.h"
