@@ -63,7 +63,7 @@
 #include "ioutil.h"
 #include "shell.h"
 #include "nstring.h"
-#include "jstring.h"
+#include "jnstring.h"
 #include "config.h"
 #include "mathfn.h"
 #include "gra.h"
@@ -170,6 +170,13 @@ int mx11loadconfig()
         if (endptr[0]=='\0') menulocal.ignorepath=val;
       }
       memfree(f1);
+    } else if (strcmp(tok,"expand_to_fullpath")==0) {
+      f1=getitok2(&s2,&len," \t,");
+      if (f1!=NULL) {
+        val=strtol(f1,&endptr,10);
+        if (endptr[0]=='\0') menulocal.expandtofullpath=val;
+      }
+      memfree(f1);
     } else if (strcmp(tok,"save_path")==0) {
       f1=getitok2(&s2,&len," \t,");
       if (f1!=NULL) {
@@ -577,6 +584,12 @@ int mx11loadconfig()
       val=strtol(f1,&endptr,10);
       if (endptr[0]=='\0') mxlocal->minus_hyphen=val;
       memfree(f1);
+    } else if (strcmp(tok,"font_slant")==0) {
+      f1=getitok2(&s2,&len," \t,");
+      val=strtol(f1,&endptr,10);
+      if ((endptr[0]=='\0') && (0<=val) && (val<=60))
+        mxlocal->font_slant=val;
+      memfree(f1);
     }
     memfree(tok);
     memfree(str);
@@ -859,6 +872,7 @@ int menuinit(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
   menulocal.expanddir=(char *)memalloc(3);
   strcpy(menulocal.expanddir,"./");
   menulocal.ignorepath=0;
+  menulocal.expandtofullpath=TRUE;
   menulocal.savepath=0;
   menulocal.savewithdata=0;
   menulocal.savewithmerge=0;
@@ -881,6 +895,7 @@ int menuinit(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
   mxlocal->cdepth=X11COLORDEPTH;
   mxlocal->backingstore=FALSE;
   mxlocal->minus_hyphen=TRUE;
+  mxlocal->font_slant=15;
   if (_putobj(obj,"_local",inst,mxlocal)) goto errexit;
   if (mx11loadconfig()) goto errexit; 
   if (exwinloadconfig()) goto errexit;
@@ -1273,6 +1288,7 @@ int mxloadfont(Display *disp,GC gc,char *fontalias,int size,int dir,
   XFontStruct *newfontstruct;
   int store;
   int twobyte=FALSE,type;
+  double ftan;
 
   fontcashfind=-1;
   for (i=0;i<mxlocal->loadfont;i++) {
@@ -1320,9 +1336,10 @@ int mxloadfont(Display *disp,GC gc,char *fontalias,int size,int dir,
   fontname2[fsp-fontname+1]='\0';
   fontname3[fsp-fontname+1]='\0';
   if ((type==ITALIC) || (type==BOLDITALIC)) {
+    ftan=tan(mxlocal->font_slant*MPI/180.0);
     sprintf(fontmatrix,"[%.3e %.3e %.3e %.3e]",
             size*fcos,size*fsin,
-           -size*fsin+size*fcos*0.36,size*fcos+size*fsin*0.36);
+           -size*fsin+size*fcos*ftan,size*fcos+size*fsin*ftan);
   } else {
     sprintf(fontmatrix,"[%.3e %.3e %.3e %.3e]",
             size*fcos,size*fsin,-size*fsin,size*fcos);
@@ -1370,9 +1387,11 @@ int mxloadfont(Display *disp,GC gc,char *fontalias,int size,int dir,
     (mxlocal->font[store]).fonttype=type;
     (mxlocal->font[store]).fontdir=dir;
     if (strcmp(fname2[0]+strlen(fname2[0])-9,"iso8859-1")==0)
-      (mxlocal->font[store]).iso8859=TRUE;
+      (mxlocal->font[store]).iso=ISO8859;
+    else if (strcmp(fname2[0]+strlen(fname2[0])-10,"iso10646-1")==0)
+      (mxlocal->font[store]).iso=ISO10646;
     else
-      (mxlocal->font[store]).iso8859=FALSE;
+      (mxlocal->font[store]).iso=ISOOTHER;
     if (top) XSetFont(disp,gc,(mxlocal->font[store]).font);
     mxlocal->loadfont++;
     if (fname2!=NULL) XFreeFontNames(fname2);
@@ -1384,6 +1403,38 @@ int mxloadfont(Display *disp,GC gc,char *fontalias,int size,int dir,
   return -1;
 }
 
+void mxXDrawString(struct fontlocal font,Display *Disp,Drawable win,GC gc,
+                  int x,int y,char *ch)
+{
+  XChar2b kanji[1];
+
+  if ((font.iso==ISO10646) && (ch[0]=='-')) {
+    kanji[0].byte1=0x22;
+    kanji[0].byte2=0x12;
+    XDrawString16(Disp,win,gc,x,y,kanji,1);
+  } else {
+    XDrawString(Disp,win,gc,x,y,ch,1);
+  }
+}
+
+int mxXTextWidth(struct fontlocal font,int minus_hyphen,char *ch)
+{
+  XChar2b kanji[1];
+
+  if (font.fontstruct==NULL) {
+      return font.fontsize;
+  } else if ((font.iso==ISO10646) && (ch[0]=='-')) {
+    kanji[0].byte1=0x22;
+    kanji[0].byte2=0x12;
+    return XTextWidth16(font.fontstruct,kanji,1);
+  } else {
+    if ((font.iso==ISO8859) && minus_hyphen && (ch[0]=='-')) {
+      ch[0]='+';
+    }
+    return XTextWidth(font.fontstruct,ch,1);
+  }
+}
+
 int mx_output(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
 {
   char code;
@@ -1628,31 +1679,23 @@ int mx_output(struct objlist *obj,char *inst,char *rval,int argc,char **argv)
         ch[0]=cstr[i];
         i++;
       }
-      XDrawString(Disp,mxlocal->win,mxlocal->gc,
-                  mxd2px(nround(x0)),
-                  mxd2py(nround(y0)),ch,1); 
+      mxXDrawString(mxlocal->font[0],Disp,mxlocal->win,mxlocal->gc,
+                    mxd2px(nround(x0)),
+                    mxd2py(nround(y0)),ch);
       if ((mxlocal->font[0].fonttype==BOLD) 
        || (mxlocal->font[0].fonttype==BOLDITALIC)) {
         bs=mxlocal->fontsize*0.018;
-        XDrawString(Disp,mxlocal->win,mxlocal->gc,
+        mxXDrawString(mxlocal->font[0],Disp,mxlocal->win,mxlocal->gc,
           mxd2px(nround(x0+bs)),
-          mxd2py(nround(y0)),ch,1); 
-        XDrawString(Disp,mxlocal->win,mxlocal->gc,
+          mxd2py(nround(y0)),ch); 
+        mxXDrawString(mxlocal->font[0],Disp,mxlocal->win,mxlocal->gc,
           mxd2px(nround(x0)),
-          mxd2py(nround(y0-bs)),ch,1); 
-        XDrawString(Disp,mxlocal->win,mxlocal->gc,
+          mxd2py(nround(y0-bs)),ch); 
+        mxXDrawString(mxlocal->font[0],Disp,mxlocal->win,mxlocal->gc,
           mxd2px(nround(x0+bs)),
-          mxd2py(nround(y0-bs)),ch,1); 
-      }
-      if ((mxlocal->font[0]).fontstruct==NULL) {
-        fontwidth=mxp2d((mxlocal->font[0]).fontsize);
-      } else {
-        if ((mxlocal->font[0]).iso8859
-        && mxlocal->minus_hyphen && (ch[0]=='-')) {
-          ch[0]='+';
-        }
-        fontwidth=mxp2d(XTextWidth((mxlocal->font[0]).fontstruct,ch,1));
+          mxd2py(nround(y0-bs)),ch); 
       }
+      fontwidth=mxp2d(mxXTextWidth(mxlocal->font[0],mxlocal->minus_hyphen,ch));
       x0+=(fontwidth+mxlocal->fontspace)*mxlocal->fontcos;
       y0-=(fontwidth+mxlocal->fontspace)*mxlocal->fontsin; 
     }
@@ -1735,11 +1778,8 @@ int mx_charwidth(struct objlist *obj,char *inst,char *rval,
         kanji[0].byte2=jis & 0xff;
         *(int *)rval=mxp2d(XTextWidth16(fontstruct,kanji,1));
       } else {
-        if ((mxlocal->font[cashpos]).iso8859 && mxlocal->minus_hyphen
-        && (ch[0]=='-')) {
-          ch[0]='+';
-        }
-        *(int *)rval=mxp2d(XTextWidth(fontstruct,ch,1));
+        *(int *)rval=mxp2d(mxXTextWidth(mxlocal->font[cashpos],
+                                        mxlocal->minus_hyphen,ch));
       }
     }
   } else *(int *)rval=nround(size*0.600);
