@@ -58,6 +58,10 @@
 #define ERRCMAP 101
 #define ERRFONT 102
 
+#define ISOOTHER 0
+#define ISO8859 1
+#define ISO10646 2
+
 extern int globallock;
 extern struct savedstdio x11iosave;
 
@@ -96,7 +100,7 @@ struct fontlocal {
   Font font;
   XFontStruct *fontstruct;
   int fontsize,fonttype,fontdir;
-  int iso8859;
+  int iso;
 };
 
 struct mxlocal {
@@ -122,6 +126,7 @@ struct mxlocal {
   int linetonum;
   int lock;
   int minus_hyphen;
+  int font_slant;
 };
 
 struct menulocal {
@@ -157,6 +162,7 @@ struct menulocal {
   char *expanddir;
   int expand;
   int ignorepath;
+  int expandtofullpath;
   int changedirectory;
   int savehistory;
   int savepath,savewithdata,savewithmerge;
