@@ -1,21 +1,41 @@
-/**
- *
- * $Id: x11gui.c,v 1.3 1999/04/11 06:09:20 isizaka Exp $
- *
- * Copyright (C) 1997, Satoshi ISHIZAKA.
- *
- * This file is part of Ngraph
- *
- * This is free software; you can redistribute it and/or modify it.
- * However, it is prohibited to compile this on the "Windows" environment.
- *
- * Original author: Satoshi ISHIZAKA
- *                  isizaka@msa.biglobe.ne.jp
- **/
+/* 
+ * $Id: x11gui.c,v 1.7 2003/02/16 12:43:37 isizaka Exp isizaka $
+ * 
+ * This file is part of "Ngraph for X11".
+ * 
+ * Copyright (C) 2002, Satoshi ISHIZAKA. isizaka@msa.biglobe.ne.jp
+ * 
+ * "Ngraph for X11" is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ * 
+ * "Ngraph for X11" is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * 
+ */
 
 /**
  *
  * $Log: x11gui.c,v $
+ * Revision 1.7  2003/02/16 12:43:37  isizaka
+ * for release 6.13.18
+ *
+ * Revision 1.6  2002/07/06 08:57:25  isizaka
+ * change to GPL.
+ *
+ * Revision 1.5  2001/07/14 17:44:50  isizaka
+ * for 6.03.14
+ *
+ * Revision 1.4  2001/03/23 12:17:43  isizaka
+ * for 6.3.13
+ *
  * Revision 1.3  1999/04/11 06:09:20  isizaka
  * *** empty log message ***
  *
@@ -38,6 +58,7 @@
 #include "x11gui.h"
 
 extern XtAppContext Application;
+void ResetEvent();
 
 struct nGetOpenFileData {
   Widget parent,widget;
@@ -62,7 +83,7 @@ struct ColorDialog {
   void (*CloseWindow)(Widget w,void *data);
   int R,G,B;
   Widget RS,GS,BS;
-  Widget CB[8]; 
+  Widget CB[24]; 
 } ColorSelection={NULL,NULL};
 
 struct DialogType {
@@ -534,7 +555,7 @@ void ColorDialogCB(Widget w,XtPointer client_data,XtPointer call_data)
   int i;
 
   d=&ColorSelection;
-  for (i=0;i<8;i++) if (w==d->CB[i]) break;
+  for (i=0;i<24;i++) if (w==d->CB[i]) break;
   switch (i) {
   case 0:
     d->R=0; d->G=0; d->B=0;
@@ -560,6 +581,54 @@ void ColorDialogCB(Widget w,XtPointer client_data,XtPointer call_data)
   case 7:
     d->R=255; d->G=255; d->B=255;
     break;
+  case 8:
+    d->R=63; d->G=63; d->B=63;
+    break;
+  case 9:
+    d->R=127; d->G=127; d->B=255;
+    break;
+  case 10:
+    d->R=127; d->G=255; d->B=127;
+    break;
+  case 11:
+    d->R=127; d->G=255; d->B=255;
+    break;
+  case 12:
+    d->R=255; d->G=127; d->B=127;
+    break;
+  case 13:
+    d->R=255; d->G=127; d->B=255;
+    break;
+  case 14:
+    d->R=255; d->G=255; d->B=127;
+    break;
+  case 15:
+    d->R=191; d->G=191; d->B=191;
+    break;
+  case 16:
+    d->R=127; d->G=127; d->B=127;
+    break;
+  case 17:
+    d->R=0; d->G=0; d->B=127;
+    break;
+  case 18:
+    d->R=0; d->G=127; d->B=0;
+    break;
+  case 19:
+    d->R=0; d->G=127; d->B=127;
+    break;
+  case 20:
+    d->R=127; d->G=0; d->B=0;
+    break;
+  case 21:
+    d->R=127; d->G=0; d->B=127;
+    break;
+  case 22:
+    d->R=127; d->G=127; d->B=0;
+    break;
+  case 23:
+    d->R=127; d->G=127; d->B=127;
+    break;
   }
   XtVaSetValues(XtNameToWidget(d->widget,"*R"),XmNvalue,d->R,NULL);
   XtVaSetValues(XtNameToWidget(d->widget,"*G"),XmNvalue,d->G,NULL);
@@ -572,7 +641,7 @@ void ColorDialogSetup(Widget w,void *data,int makewidget)
 {
   Arg al[20];
   Cardinal ac;
-  Widget rc,rc2,frame;
+  Widget rc,rc2,rc3,urc,frame;
   struct ColorDialog *d;
   int i;
   
@@ -581,36 +650,110 @@ void ColorDialogSetup(Widget w,void *data,int makewidget)
   ac=0;
   XtSetArg(al[ac],XmNorientation,XmVERTICAL); ac++;
   XtManageChild(rc=XmCreateRowColumn(w,"rc",al,ac));
-  ac=0;
-  XtManageChild(frame=XmCreateFrame(rc,"buttonf",al,ac));
+
   ac=0;
   XtSetArg(al[ac],XmNorientation,XmHORIZONTAL); ac++;
+  XtManageChild(urc=XmCreateRowColumn(rc,"urc",al,ac));
+
+  ac=0;
+  XtManageChild(frame=XmCreateFrame(urc,"buttonf",al,ac));
+  ac=0;
+  XtSetArg(al[ac],XmNorientation,XmVERTICAL); ac++;
   XtManageChild(rc2=XmCreateRowColumn(frame,"buttonrc",al,ac));
+
+  ac=0;
+  XtSetArg(al[ac],XmNorientation,XmHORIZONTAL); ac++;
+  XtManageChild(rc3=XmCreateRowColumn(rc2,"buttonrc1",al,ac));
   ac=0;
   XtSetArg(al[ac],XmNbackground,RGB(0,0,0)); ac++;
-  XtManageChild(d->CB[0]=XmCreatePushButton(rc2,"   ",al,ac));
-  ac=0;
+  XtManageChild(d->CB[0]=XmCreatePushButton(rc3,"   ",al,ac));
   ac=0;
   XtSetArg(al[ac],XmNbackground,RGB(0,0,255)); ac++;
-  XtManageChild(d->CB[1]=XmCreatePushButton(rc2,"   ",al,ac));
+  XtManageChild(d->CB[1]=XmCreatePushButton(rc3,"   ",al,ac));
   ac=0;
   XtSetArg(al[ac],XmNbackground,RGB(0,255,0)); ac++;
-  XtManageChild(d->CB[2]=XmCreatePushButton(rc2,"   ",al,ac));
+  XtManageChild(d->CB[2]=XmCreatePushButton(rc3,"   ",al,ac));
   ac=0;
   XtSetArg(al[ac],XmNbackground,RGB(0,255,255)); ac++;
-  XtManageChild(d->CB[3]=XmCreatePushButton(rc2,"   ",al,ac));
+  XtManageChild(d->CB[3]=XmCreatePushButton(rc3,"   ",al,ac));
   ac=0;
   XtSetArg(al[ac],XmNbackground,RGB(255,0,0)); ac++;
-  XtManageChild(d->CB[4]=XmCreatePushButton(rc2,"   ",al,ac));
+  XtManageChild(d->CB[4]=XmCreatePushButton(rc3,"   ",al,ac));
   ac=0;
   XtSetArg(al[ac],XmNbackground,RGB(255,0,255)); ac++;
-  XtManageChild(d->CB[5]=XmCreatePushButton(rc2,"   ",al,ac));
+  XtManageChild(d->CB[5]=XmCreatePushButton(rc3,"   ",al,ac));
   ac=0;
   XtSetArg(al[ac],XmNbackground,RGB(255,255,0)); ac++;
-  XtManageChild(d->CB[6]=XmCreatePushButton(rc2,"   ",al,ac));
+  XtManageChild(d->CB[6]=XmCreatePushButton(rc3,"   ",al,ac));
   ac=0;
   XtSetArg(al[ac],XmNbackground,RGB(255,255,255)); ac++;
-  XtManageChild(d->CB[7]=XmCreatePushButton(rc2,"   ",al,ac));
+  XtManageChild(d->CB[7]=XmCreatePushButton(rc3,"   ",al,ac));
+
+  ac=0;
+  XtSetArg(al[ac],XmNorientation,XmHORIZONTAL); ac++;
+  XtManageChild(rc3=XmCreateRowColumn(rc2,"buttonrc2",al,ac));
+  ac=0;
+  XtSetArg(al[ac],XmNbackground,RGB(63,63,63)); ac++;
+  XtManageChild(d->CB[8]=XmCreatePushButton(rc3,"   ",al,ac));
+  ac=0;
+  ac=0;
+  XtSetArg(al[ac],XmNbackground,RGB(127,127,255)); ac++;
+  XtManageChild(d->CB[9]=XmCreatePushButton(rc3,"   ",al,ac));
+  ac=0;
+  XtSetArg(al[ac],XmNbackground,RGB(127,255,127)); ac++;
+  XtManageChild(d->CB[10]=XmCreatePushButton(rc3,"   ",al,ac));
+  ac=0;
+  XtSetArg(al[ac],XmNbackground,RGB(127,255,255)); ac++;
+  XtManageChild(d->CB[11]=XmCreatePushButton(rc3,"   ",al,ac));
+  ac=0;
+  XtSetArg(al[ac],XmNbackground,RGB(255,127,127)); ac++;
+  XtManageChild(d->CB[12]=XmCreatePushButton(rc3,"   ",al,ac));
+  ac=0;
+  XtSetArg(al[ac],XmNbackground,RGB(255,127,255)); ac++;
+  XtManageChild(d->CB[13]=XmCreatePushButton(rc3,"   ",al,ac));
+  ac=0;
+  XtSetArg(al[ac],XmNbackground,RGB(255,255,127)); ac++;
+  XtManageChild(d->CB[14]=XmCreatePushButton(rc3,"   ",al,ac));
+  ac=0;
+  XtSetArg(al[ac],XmNbackground,RGB(191,191,191)); ac++;
+  XtManageChild(d->CB[15]=XmCreatePushButton(rc3,"   ",al,ac));
+
+  ac=0;
+  XtSetArg(al[ac],XmNorientation,XmHORIZONTAL); ac++;
+  XtManageChild(rc3=XmCreateRowColumn(rc2,"buttonrc3",al,ac));
+  ac=0;
+  XtSetArg(al[ac],XmNbackground,RGB(127,127,127)); ac++;
+  XtManageChild(d->CB[16]=XmCreatePushButton(rc3,"   ",al,ac));
+  ac=0;
+  ac=0;
+  XtSetArg(al[ac],XmNbackground,RGB(0,0,127)); ac++;
+  XtManageChild(d->CB[17]=XmCreatePushButton(rc3,"   ",al,ac));
+  ac=0;
+  XtSetArg(al[ac],XmNbackground,RGB(0,127,0)); ac++;
+  XtManageChild(d->CB[18]=XmCreatePushButton(rc3,"   ",al,ac));
+  ac=0;
+  XtSetArg(al[ac],XmNbackground,RGB(0,127,127)); ac++;
+  XtManageChild(d->CB[19]=XmCreatePushButton(rc3,"   ",al,ac));
+  ac=0;
+  XtSetArg(al[ac],XmNbackground,RGB(127,0,0)); ac++;
+  XtManageChild(d->CB[20]=XmCreatePushButton(rc3,"   ",al,ac));
+  ac=0;
+  XtSetArg(al[ac],XmNbackground,RGB(127,0,127)); ac++;
+  XtManageChild(d->CB[21]=XmCreatePushButton(rc3,"   ",al,ac));
+  ac=0;
+  XtSetArg(al[ac],XmNbackground,RGB(127,127,0)); ac++;
+  XtManageChild(d->CB[22]=XmCreatePushButton(rc3,"   ",al,ac));
+  ac=0;
+  XtSetArg(al[ac],XmNbackground,RGB(127,127,127)); ac++;
+  XtManageChild(d->CB[23]=XmCreatePushButton(rc3,"   ",al,ac));
+
+  ac=0;
+  XtManageChild(frame=XmCreateFrame(urc,"colorf",al,ac));
+  ac=0;
+  XtSetArg(al[ac],XmNrecomputeSize,FALSE); ac++;
+  XtSetArg(al[ac],XmNlabelType,XmPIXMAP); ac++;
+  XtManageChild(XmCreateLabel(frame,"color",al,ac));
+
   ac=0;
   XtSetArg(al[ac],XmNorientation,XmHORIZONTAL); ac++;
   XtManageChild(rc2=XmCreateRowColumn(rc,"rrc",al,ac));
@@ -644,16 +787,11 @@ void ColorDialogSetup(Widget w,void *data,int makewidget)
   XtSetArg(al[ac],XmNorientation,XmHORIZONTAL); ac++;
   XtSetArg(al[ac],XmNshowValue,XmNEAR_SLIDER); ac++;
   XtManageChild(d->BS=XmCreateScale(rc2,"B",al,ac));
-  ac=0;
-  XtManageChild(frame=XmCreateFrame(rc,"colorf",al,ac));
-  ac=0;
-  XtSetArg(al[ac],XmNrecomputeSize,FALSE); ac++;
-  XtSetArg(al[ac],XmNlabelType,XmPIXMAP); ac++;
-  XtManageChild(XmCreateLabel(frame,"color",al,ac));
+
   XtAddCallback(d->RS,XmNvalueChangedCallback,ColorDialogScale,d);
   XtAddCallback(d->GS,XmNvalueChangedCallback,ColorDialogScale,d);
   XtAddCallback(d->BS,XmNvalueChangedCallback,ColorDialogScale,d);
-  for (i=0;i<8;i++)
+  for (i=0;i<24;i++)
     XtAddCallback(d->CB[i],XmNdisarmCallback,ColorDialogCB,d);
   }
   XtVaSetValues(XtNameToWidget(w,"*R"),XmNvalue,d->R,NULL);
