@@ -1,6 +1,6 @@
 /**
  *
- * $Id: gra2wmf.c,v 1.1 2003/08/15 12:56:18 isizaka Exp isizaka $
+ * $Id: gra2wmf.c,v 1.2 2004/10/30 15:28:11 isizaka Exp isizaka $
  *
  * This is free software; you can redistribute it and/or modify it.
  *
@@ -11,6 +11,9 @@
 /**
  *
  * $Log: gra2wmf.c,v $
+ * Revision 1.2  2004/10/30 15:28:11  isizaka
+ * bug fix: width of jpn font, thanks to ka-tana (tanaka@bp.jp.nec.com)
+ *
  * Revision 1.1  2003/08/15 12:56:18  isizaka
  * Initial revision
  *
@@ -41,7 +44,7 @@
 #define CONFSEP "/"
 #define NSTRLEN 256
 #define LINETOLIMIT 500
-#define VERSION "1.00.00"
+#define VERSION "1.00.01"
 
 #define TRUE  1
 #define FALSE 0
@@ -334,7 +337,7 @@ errexit:
   return FALSE;
 }
 
-int round(double x)
+int nround(double x)
 {
   int ix;
   double dx;
@@ -773,7 +776,6 @@ void getboundingbox(char code,int *cpar,char *cstr)
     i=0;
     while (i<strlen(cstr)) {
       if (niskanji((unsigned char)cstr[i]) && (cstr[i+1]!='\0')) {
-        i+=2;
         w=charwidth((((unsigned char)cstr[i+1])<<8)+(unsigned char)cstr[i],
                        bfontalias,bpt);
         h=charheight(bfontalias,bpt);
@@ -798,6 +800,7 @@ void getboundingbox(char code,int *cpar,char *cstr)
         setbbminmax(x2,y2,x3,y3,FALSE);
         bposx+=(int )((w+bspc*25.4/72)*ccos);
         bposy-=(int )((w+bspc*25.4/72)*csin);
+        i+=2;
       } else i++;
     }
     break;
@@ -831,8 +834,8 @@ void lineto(HDC DC,int x,int y)
     len2=len=sqrt(dx*dx+dy*dy);
     while (len2>((dashlist)[dashi]-dashlen)) {
       dd=(len-len2+(dashlist)[dashi]-dashlen)/len;
-      gx=gx1+round(dx*dd);
-      gy=gy1+round(dy*dd);
+      gx=gx1+nround(dx*dd);
+      gy=gy1+nround(dy*dd);
       if (dotf) LineTo(DC,gx,gy);
       else MoveTo(DC,gx,gy);
       dotf=dotf ? FALSE : TRUE;
@@ -853,22 +856,22 @@ void lineto(HDC DC,int x,int y)
 
 int dot2pixel(int r)
 {
-  return round(r*pixel_dot);
+  return nround(r*pixel_dot);
 }
 
 int dot2pixelx(int x)
 {
-  return round(x*pixel_dot+offsetx-scrollx);
+  return nround(x*pixel_dot+offsetx-scrollx);
 }
 
 int dot2pixely(int y)
 {
-  return round(y*pixel_dot+offsety-scrolly);
+  return nround(y*pixel_dot+offsety-scrolly);
 }
 
 int pixel2dot(int r)
 {
-  return round(r/pixel_dot);
+  return nround(r/pixel_dot);
 }
 
 void draw(char code,int *cpar,char *cstr)
@@ -1018,10 +1021,10 @@ void draw(char code,int *cpar,char *cstr)
                dot2pixely(cpar[2]-cpar[4]),
                dot2pixelx(cpar[1]+cpar[3]),
                dot2pixely(cpar[2]+cpar[4]),
-               dot2pixelx(cpar[1]+round(cpar[3]*cos(Theta1)))-1,
-               dot2pixely(cpar[2]-round(cpar[4]*sin(Theta1)))-1,
-               dot2pixelx(cpar[1]+round(cpar[3]*cos(Theta2)))-1,
-               dot2pixely(cpar[2]-round(cpar[4]*sin(Theta2)))-1);
+               dot2pixelx(cpar[1]+nround(cpar[3]*cos(Theta1)))-1,
+               dot2pixely(cpar[2]-nround(cpar[4]*sin(Theta1)))-1,
+               dot2pixelx(cpar[1]+nround(cpar[3]*cos(Theta2)))-1,
+               dot2pixely(cpar[2]-nround(cpar[4]*sin(Theta2)))-1);
       } else {
         TmpPen=OrgPen;
         SelectObject(DC,TmpPen);
@@ -1033,19 +1036,19 @@ void draw(char code,int *cpar,char *cstr)
                  dot2pixely(cpar[2]-cpar[4]),
                  dot2pixelx(cpar[1]+cpar[3]),
                  dot2pixely(cpar[2]+cpar[4]),
-                 dot2pixelx(cpar[1]+round(cpar[3]*cos(Theta1)))-1,
-                 dot2pixely(cpar[2]-round(cpar[4]*sin(Theta1)))-1,
-                 dot2pixelx(cpar[1]+round(cpar[3]*cos(Theta2)))-1,
-                 dot2pixely(cpar[2]-round(cpar[4]*sin(Theta2)))-1);
+                 dot2pixelx(cpar[1]+nround(cpar[3]*cos(Theta1)))-1,
+                 dot2pixely(cpar[2]-nround(cpar[4]*sin(Theta1)))-1,
+                 dot2pixelx(cpar[1]+nround(cpar[3]*cos(Theta2)))-1,
+                 dot2pixely(cpar[2]-nround(cpar[4]*sin(Theta2)))-1);
         } else {
           Chord(DC,dot2pixelx(cpar[1]-cpar[3]),
                    dot2pixely(cpar[2]-cpar[4]),
                    dot2pixelx(cpar[1]+cpar[3]),
                    dot2pixely(cpar[2]+cpar[4]),
-                   dot2pixelx(cpar[1]+round(cpar[3]*cos(Theta1)))-1,
-                   dot2pixely(cpar[2]-round(cpar[4]*sin(Theta1)))-1,
-                   dot2pixelx(cpar[1]+round(cpar[3]*cos(Theta2)))-1,
-                   dot2pixely(cpar[2]-round(cpar[4]*sin(Theta2)))-1);
+                   dot2pixelx(cpar[1]+nround(cpar[3]*cos(Theta1)))-1,
+                   dot2pixely(cpar[2]-nround(cpar[4]*sin(Theta1)))-1,
+                   dot2pixelx(cpar[1]+nround(cpar[3]*cos(Theta2)))-1,
+                   dot2pixely(cpar[2]-nround(cpar[4]*sin(Theta2)))-1);
         }
         SelectObject(DC,ThePen);
       }
@@ -1129,9 +1132,9 @@ void draw(char code,int *cpar,char *cstr)
       loadfontf=FALSE;
       break;
     }
-    IDFont.lfHeight=-round(pixel_dot*fontsize);
+    IDFont.lfHeight=-nround(pixel_dot*fontsize);
     IDFont.lfWidth=0;
-    IDFont.lfEscapement=IDFont.lfOrientation=round(fontdir*10);
+    IDFont.lfEscapement=IDFont.lfOrientation=nround(fontdir*10);
     IDFont.lfUnderline=0;
     IDFont.lfStrikeOut=0;
     IDFont.lfPitchAndFamily=0;
@@ -1199,8 +1202,8 @@ void draw(char code,int *cpar,char *cstr)
     if (s==NULL) break;
     x0=cpx;
     y0=cpy;
-    TextOut(DC,dot2pixelx(round(x0)),
-               dot2pixely(round(y0)),s,strlen(s));
+    TextOut(DC,dot2pixelx(nround(x0)),
+               dot2pixely(nround(y0)),s,strlen(s));
     fontwidth=0;
     for (i=0;i<strlen(s);i++) {
       fontwidth+=charwidth((unsigned char)s[i],fontalias,fontsize/25.4*72);
@@ -1209,25 +1212,25 @@ void draw(char code,int *cpar,char *cstr)
     free(s);
     x0+=fontwidth*fontcos;
     y0-=fontwidth*fontsin;
-    cpx=round(x0);
-    cpy=round(y0);
+    cpx=nround(x0);
+    cpy=nround(y0);
     break;
   case 'K':
     if (!loadfontf) break;
     x0=cpx;
     y0=cpy;
-    TextOut(DC,dot2pixelx(round(x0)),
-               dot2pixely(round(y0)),cstr,strlen(cstr));
+    TextOut(DC,dot2pixelx(nround(x0)),
+               dot2pixely(nround(y0)),cstr,strlen(cstr));
     fontwidth=0;
-    for (i=0;i<strlen(s);i+=2) {
+    for (i=0;i<strlen(cstr);i+=2) {
       fontwidth+=charwidth((((unsigned char)cstr[i+1])<<8)+(unsigned char)cstr[i],
                            fontalias,fontsize/25.4*72);
       fontwidth+=fontspace;
     }
     x0+=fontwidth*fontcos;
     y0-=fontwidth*fontsin;
-    cpx=round(x0);
-    cpy=round(y0);
+    cpx=nround(x0);
+    cpy=nround(y0);
     break;
   default: break;
   }
